<?php
/**
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2018 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

include_once _PS_MODULE_DIR_.'smartpay/smartpay.php';

class SmartPayValidationModuleFrontController extends ModuleFrontController
{
    public function __construct()
    {
        parent::__construct();
        $this->ssl = true;
        $this->context = Context::getContext();
    }

    public function postProcess()
    {
        $key    = Configuration::get('SMART_PAY_KEY');
        $secret = Configuration::get('SMART_PAY_SECRET');
        $display = new FrontController();

        /* we are returing form payment gateway */
        if (Tools::getIsset('sc_transaction_status') && Tools::getIsset('sc_order_id')) {

            $sc_transaction_status = Tools::getValue('sc_transaction_status');
            $sc_order_id = Tools::getValue('sc_order_id');
            $sc_cart_id = Tools::getValue('sc_cart_id');
            $sc_transaction_token = Tools::getValue('sc_transaction_token');
            $cart = $this->context->cart;
            $customer = new Customer($cart->id_customer);
            
            //check smartcash


        $confirmActionUrl            = "https://www.smart-cash.it/smartPay/getTransactionStatus.php";
        $params['merchantUsername']  = $key;
        $params['merchantPassword']  = $secret;
        $params['transactionToken']  = $sc_transaction_token;
        $params['transactionCartId'] = $sc_cart_id;
        
        $confirmActionUrl .= "?" . http_build_query($params);

        $apiResult = file_get_contents($confirmActionUrl);


         if (!$apiResult) {
            
            $_GET['content_only'] = 1;
            $this->context->smarty->assign('error_type', 2);
            $this->context->smarty->assign('back_href', $this->context->link->getPageLink('order', true, NULL, "step=3"));
            $this->context->smarty->assign('error_msg',"Impossibile contattare il server di pagamento.");

            $this->context->smarty->display(_PS_MODULE_DIR_ . 'smartpay/views/templates/front/error.tpl');
            exit;
        }
        
        $response = json_decode($apiResult, 1);
        
        if (!$response) {
            
            $_GET['content_only'] = 1;
            $this->context->smarty->assign('error_type', 3);
            $this->context->smarty->assign('back_href', $this->context->link->getPageLink('order', true, NULL, "step=3"));
            $this->context->smarty->assign('error_msg',"Impossibile decodificare la risposta del server di pagamento.");

            $this->context->smarty->display(_PS_MODULE_DIR_ . 'smartpay/views/templates/front/error.tpl');
            exit;
        }
        
        //response taken
        
        if ($response['response']['payed'] == 0) {
            
            $_GET['content_only'] = 1;
            $this->context->smarty->assign('error_type', 4);
            $this->context->smarty->assign('error_msg', print_r($response['response']['error'],1));
            $this->context->smarty->assign('back_href', $this->context->link->getPageLink('order', true, NULL, "step=3"));
            $this->context->smarty->display(_PS_MODULE_DIR_ . 'smartpay/views/templates/front/error.tpl');
            exit;
        }else{

             try {
                        
                            $redirect = pSQL(Configuration::get('SMART_PAY_SUCCESS_REDIRECT_URL'));

                            if($redirect && !empty($redirect)){
                                        die("<script>window.location.href = '".$redirect."';</script>");
                            }else{
                            $this->redirectConfirmation(
                                $this->module->id,
                                $sc_cart_id,
                                (int)$this->module->currentOrder,
                                $customer->secure_key,
                                $sc_transaction_token
                            );
                        }


                       
                    } catch (PrestaShopException $e) {


                    }

                }


         
        } else{            $this->redirectFailedPayment($this->module->l('Technical error on gateway response.
                                                           Please check if the order was added. Error code: ', 'validation') . '003');
        }
        
    }

    private function redirectFailedPayment($error = '')
    {
        if (Configuration::get('PS_ORDER_PROCESS_TYPE')) {
            Tools::redirect('index.php?controller=order-opc&isPaymentStep=true&error_msg='.urlencode($error));
        } else {
            Tools::redirect('index.php?controller=order&step=3&error_msg='.urlencode($error));
        }
    }

    private function redirectConfirmation($id_module, $id_cart, $id_order, $secure_key,$token = "")
    {
       
        Tools::redirect('index.php?controller=order-confirmation&id_cart='.$id_cart.
                        '&id_module='.$id_module.'&id_order='.$id_order.'&key='.$secure_key.'&sc_transaction_token='.$token);
    }
}
