<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2018 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

global $kernel;
    if(!$kernel){ 
      require_once _PS_ROOT_DIR_.'/app/AppKernel.php';
      $kernel = new \AppKernel('prod', false);
      $kernel->boot(); 
    }

class SmartPay extends PaymentModule
{
    protected $js_path = null;
    protected $css_path = null;
    public $api = null;
    
    public function __construct()
    {
        
        $this->name                   = 'smartpay';
        $this->tab                    = 'payments_gateways';
        $this->version                = '1.1.8';
        $this->author                 = 'Stonewall Capital SRL';
        $this->ps_versions_compliancy = array(
            'min' => '1.7',
            'max' => _PS_VERSION_
        );
        
        $this->currencies       = true;
        $this->currencies_mode  = 'checkbox';
        $this->bootstrap        = true;
        $this->need_instance    = false;
        $this->is_eu_compatible = true;
        
        parent::__construct();
        
        $this->displayName      = $this->l('SmartPay');
        $this->description      = $this->l('The official SmartPay plugin for PrestaShop.');
        $this->confirmUninstall = $this->l('Are you sure you want to delete module data?');
        
        /* paths */
        $this->js_path        = $this->_path . 'views/js/';
        $this->css_path       = $this->_path . 'views/css/';
    }
    
    /**
     * Hook used for versions <= 1.6 to add css and js for iframe
     */
    public function hookDisplayHeader($params)
    {
        
    }
    
  
    /**
     * New hook version 1.7 payment api
     */
    public function hookPaymentOptions($params)
    {
        
        $cart = $params['cart'];
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        if (Tools::getValue('error_msg')) {
            $this->context->controller->errors[] = Tools::getValue('error_msg');
        }
        
        
        $cart_details = $params['cart']->getSummaryDetails(null, true);
        
        $order_currency = "EUR";
        $products       = $cart->getProducts();
        foreach ($products as $key => $product) {
            $products[$key]['name'] = str_replace('"', '\'', $product['name']);
            $products[$key]['name'] = htmlentities(utf8_decode($product['name']));
        }
        
        $discounts = $cart_details['discounts'];
        
        $carrier = $cart_details['carrier'];
        
        if (_PS_VERSION_ < '1.5') {
            $shipping_cost = $cart_details['total_shipping_tax_exc'];
        } else {
            $shipping_cost = $this->context->cart->getTotalShippingCost();
        }
        
        $CheckoutUrl   = 'https://www.smart-cash.it/smartPay/initializePayment.php';
        $cart_order_id = $cart->id;
        $total         = number_format($cart->getOrderTotal(true, 3), 2, '.', '');
        
        
        
        
        $params                     = array();
        $params['merchantUsername'] = Configuration::get('SMART_PAY_KEY');
        $params['merchantPassword'] = Configuration::get('SMART_PAY_SECRET');
        $params['paymentMethod']    = "SC_PAY_USING_CBK";
        
        $params['redirectTo']           = base64_encode($this->context->link->getModuleLink('smartpay', 'validation', array(), true)); //$this->context->link->getPageLink('order', true, NULL, "step=3")
        $params['s2s']           = base64_encode($this->context->link->getModuleLink('smartpay', 's2s', array(), true)); //$this->context->link->getPageLink('order', true, NULL, "step=3")
        $params['jsonCart']['cart_id']  = $cart_order_id;
        $params['jsonCart']['order_id'] = $cart_order_id;
        $params['jsonCart']['total']    = $total;
        $params['jsonCart']['currency'] = $order_currency;
        
        if ($shipping_cost > 0) {
            $params['jsonCart']['shipping'] = $shipping_cost;
        }
        if ($cart_details['total_tax'] > 0) {
            $params['jsonCart']['additional'] = $cart_details['total_tax'];
        }
        if ($cart_details['total_discounts_tax_exc'] > 0) {
            $params['jsonCart']['discount'] = $cart_details['total_discounts_tax_exc'];
        }
        
        
        
        
        
        
        
        $i           = 0;
        $imageHandle = new Image();
        $linkHandle  = new Link();
        
        foreach ($products as $product) {
            
            $image                                           = $imageHandle->getCover($product['id_product']);
            $imagePath                                       = $linkHandle->getImageLink($product['link_rewrite'], $image['id_image'], 'home_default');
            $params['jsonCart']['values'][$i]['title']       = htmlentities(strip_tags($product['name']), ENT_QUOTES);
            $params['jsonCart']['values'][$i]['description'] = htmlentities(strip_tags($product['description_short']), ENT_QUOTES);
            $params['jsonCart']['values'][$i]['quantity']    = $product['quantity'];
            $params['jsonCart']['values'][$i]['image']       = $imagePath;
            $params['jsonCart']['values'][$i]['amount']      = $product['price_wt'];
            $params['jsonCart']['values'][$i]['shipping']    = 0;
            $params['jsonCart']['values'][$i]['additional']  = 0;
            if (isset($product['id_manufacturer']) && isset($manufacturers[$product['id_manufacturer']])) {
                $params['jsonCart']['values'][$i]['employee'] = $manufacturers[$product['id_manufacturer']]->smart_cash_vendor_token;
            }
            $i++;
            unset($image);
            unset($imagePath);
        }
        
        
         
        
        $params['jsonCart'] = base64_encode(json_encode($params['jsonCart']));
            

        $authUrl = $CheckoutUrl . '?' . http_build_query($params);
        $result  = file_get_contents($authUrl);
        $result  = json_decode($result, 1);
        
        
        $errorMessage = "";
        
        if (!$result) {
            $errorMessage = ("<div style='border: 1px solid;margin: 10px 0px;padding:15px 10px 15px 50px;background-repeat: no-repeat;background-position: 10px center;color: #D8000C;background-color: #FFBABA;'>Invalid JSON<a href = '" . $this->context->link->getPageLink('order', true, NULL, "step=3") . "'>Torna indietro</a></div>");
        }
        
        
        if ($result['response']['transaction_status'] == 0) {
            $errorMessage = ("<div style='border: 1px solid;margin: 10px 0px;padding:15px 10px 15px 50px;background-repeat: no-repeat;background-position: 10px center;color: #D8000C;background-color: #FFBABA;'>" . $result['response']['transaction_message'] . ".<a href = '" . $this->context->link->getPageLink('order', true, NULL, "step=3") . "'>Torna indietro</a></div>");
        } else {
            $redirectUrl = $result['response']['transaction_action_url'];
        }
        
        
        
        
        
        $payment_options = array();
        /* set id_shop and id_lang cookie for correct re-entry */
        
        
        if($errorMessage != ""){
             die($errorMessage);
        }else{
        if ($redirectUrl) {
            $this->context->smarty->assign('smartpay_path_image', $this->_path . 'views/img/');
            $info           = $this->fetch('module:smartpay/views/templates/hook/payment_info.tpl');
            $externalOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
            $externalOption->setCallToActionText($this->l('Pay with card or other payment methods'))->setAction($redirectUrl)->setAdditionalInformation($info);
            $payment_options[] = $externalOption;
        }
    }
        return $payment_options;
        
    }
    /**
     * Hook needed by version 1.7 to load css and js for iframe
     */
    public function hookActionFrontControllerSetMedia($params)
    {
        
    }
    
    public function hookDisplayPaymentReturn($params)
    {
        
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
            $state = $params['order']->getCurrentState();
      
        
        if ($state == Configuration::get('PS_OS_PAYMENT') || $state == Configuration::get('PS_OS_OUTOFSTOCK')) {
            $this->context->smarty->assign('transaction_result', 'ok');
            return $this->display(__FILE__, 'payment_return.tpl');
        } else {
            $this->context->smarty->assign('transaction_result', 'error');
            return $this->display(__FILE__, 'payment_return.tpl');
        }
    }
    
    /**
     * Show payment info on order page version 1.6 and up
     */
    public function hookDisplayAdminOrderLeft($params)
    {
        return $this->displayPanel($params);
    }
    
    /**
     * Show payment info on order page version 1.5
     */
    public function hookAdminOrder($params)
    {
        return $this->displayPanel($params);
    }
    
    /**
     * Called when new orderslip is added, used for Sales API
     */
    public function hookActionObjectOrderSlipAddAfter($params)
    {
        
    }
    
    /**
     * Called when order status is set to void, used for Sales API
     */
    public function hookActionOrderStatusPostUpdate($params)
    {
        
    }
    
    
    
    private function displayPanel($params)
    {
        
    }
    
    
    
    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }
        /* set default values */
        Configuration::updateValue('SMART_PAY_KEY', '');
        Configuration::updateValue('SMART_PAY_SECRET', '');
        Configuration::updateValue('SMART_PAY_SUCCESS_REDIRECT_URL', '');
        
        /* register hook based on PS Version */
        if (version_compare(_PS_VERSION_, '1.7', '>')) {
            if (parent::install()) {
                return $this->installTab() && $this->registerHook('actionObjectOrderSlipAddAfter') && $this->registerHook('displayAdminOrderLeft') && $this->registerHook('actionOrderStatusPostUpdate') && $this->registerHook('actionFrontControllerSetMedia') && $this->registerHook('paymentOptions') && $this->registerHook('paymentReturn');
            }
            return false;
        }  else {
            return false;
        }
    }
    
    public function uninstall()
    {
        
        if (!Configuration::deleteByName('SMART_PAY_KEY') || !Configuration::deleteByName('SMART_PAY_SECRET') || !Configuration::deleteByName('SMART_PAY_SUCCESS_REDIRECT_URL') || !$this->uninstallTab() || !parent::uninstall()) {
            return false;
        }
        return true;
    }
    
    /**
     * Install Tab
     * @return boolean
     */
    private function installTab()
    {
        $tab             = new Tab();
        $tab->id_parent  = -1;
        $tab->class_name = 'Admin' . get_class($this);
        $tab->module     = $this->name;
        $tab->active     = 1;
        $tab->name       = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->displayName;
        }
        unset($lang);
        return $tab->add();
    }
    
    /**
     * Uninstall Tab
     * @return boolean
     */
    private function uninstallTab()
    {
        $id_tab = (int) Tab::getIdFromClassName('Admin' . get_class($this));
        if ($id_tab) {
            $tab = new Tab($id_tab);
            if (Validate::isLoadedObject($tab)) {
                return $tab->delete();
            } else {
                return false;
            }
        } else {
            return true;
        }
    }
    
 
    
    public function getContent()
    {
        /* data for ui */
        $controller_name       = 'AdminSmartPay';
        $controller_url        = $this->context->link->getAdminLink($controller_name);
        $ps_url                = Tools::usingSecureMode() ? Tools::getShopDomainSsl(true) : Tools::getShopDomain(true);
        $modules_url           = $ps_url . __PS_BASE_URI__ . 'modules/';
        $ip                    = gethostbyname($_SERVER['SERVER_NAME']);
        $current_view          = Tools::getValue('current_view');
        $request_uri           = basename($_SERVER['REQUEST_URI']);
        $api_key_error         = 0;
        $api_key_error_message = '';
        $config_error          = 0;
        $config_error_message  = '';
        
        /* Forms submit manager  */
        $submit_message = array();
        if (Tools::isSubmit('submitConfig')) {
            Configuration::updateValue('SMART_PAY_KEY', pSQL(Tools::getValue('SMART_PAY_KEY')));
            Configuration::updateValue('SMART_PAY_SECRET', pSQL(Tools::getValue('SMART_PAY_SECRET')));
            Configuration::updateValue('SMART_PAY_SUCCESS_REDIRECT_URL', pSQL(Tools::getValue('SMART_PAY_SUCCESS_REDIRECT_URL')));
            $submit_message['config'] = $this->displayConfirmation($this->l('Configuration updated.'));
        }
        /* Test config  */
        if ($current_view == 'config' or $current_view == '') {
            
            
        }
        
        /* Test api */
        $this->context->smarty->assign(array(
            'submit_message' => $submit_message,
            'ps_url' => $ps_url,
            'modules_url' => $modules_url,
            'request_uri' => $request_uri,
            'controller_url' => $controller_url,
            'controller_name' => $controller_name,
            'config_error' => $config_error,
            'config_error_message' => $config_error_message,
            'smart_pay_key' => pSQL(Configuration::get('SMART_PAY_KEY')),
            'smart_pay_secret' => pSQL(Configuration::get('SMART_PAY_SECRET')),
            'smart_pay_success_redirect_url' => pSQL(Configuration::get('SMART_PAY_SUCCESS_REDIRECT_URL'))


        ));
        $display = $this->display(__FILE__, 'views/templates/admin/configure.tpl');
        return $display;
    }
    
    
    
    
    public function checkCurrency($cart)
    {
        $currency_order    = new Currency((int) $cart->id_currency);
        $currencies_module = $this->getCurrency((int) $cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }
    
    
}
